// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancycharsetconverter.h 909 2011-05-27 21:20:49Z fancylibs $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** Class for charset conversions.
**
*****************************************************************************/

#ifndef FANCYBASE_CHARSET_CONVERTER_H
#define FANCYBASE_CHARSET_CONVERTER_H

#include <string>
#include <fancyobject.h>

namespace Fancy {

  namespace impl {
    class CharSetConverterImpl;
  }

  /**
   * A CharSetConverter provides easy charset conversions via iconv.
   * It is not threadsafe because the iconv context is stateful, see iconv_open(3).
   */
  class FANCYBASE_API CharSetConverter
  {
  public:
    CharSetConverter(const char *to, const char *from);
    ~CharSetConverter();

    std::string convert(const std::string &in) const;

    CharSetConverter(const CharSetConverter &a);
    CharSetConverter &operator=(const CharSetConverter &a);
  private:
    ObjPtr<impl::CharSetConverterImpl> d;
  };

  class FANCYBASE_API CharToWCharSetConverter
   {
   public:
     CharToWCharSetConverter(const char *cs);
     ~CharToWCharSetConverter();

     std::wstring convert(const std::string &in) const;

     CharToWCharSetConverter(const CharToWCharSetConverter &a);
     CharToWCharSetConverter &operator=(const CharToWCharSetConverter &a);
   private:
     ObjPtr<impl::CharSetConverterImpl> d;
   };

  class FANCYBASE_API WCharToCharSetConverter
   {
   public:
     WCharToCharSetConverter(const char *cs);
     ~WCharToCharSetConverter();

     std::string convert(const std::wstring &in) const;

     WCharToCharSetConverter(const WCharToCharSetConverter &a);
     WCharToCharSetConverter &operator=(const WCharToCharSetConverter &a);
   private:
     ObjPtr<impl::CharSetConverterImpl> d;
   };

}

#endif // ! FANCYBASE_CHARSET_CONVERTER_H
