// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancyconfig.h 909 2011-05-27 21:20:49Z fancylibs $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** Class for parsing command line arguments and config files.
**
*****************************************************************************/

#ifndef FANCYBASE_CONFIG_H
#define FANCYBASE_CONFIG_H

#include <fancyobject.h>

namespace Fancy {

  class FANCYBASE_API ConfigOptions: virtual public Object
  {
  public:
      typedef ObjPtr<ConfigOptions> ptr_t;

      static ptr_t newInstance(const std::string &ctxt);

      /**
       * Loads configuration options from a text file with name=value lines.
       *
       * If I/O errors occur or if more than syntaxErrorsAllowed lines are malformed, a Fancy::Exception is thrown.
       *
       * Empty lines are allowed.
       * White space is stripped basically, but is respected within names or values.
       * Lines with # or ; as first non-white character are considered comments.
       */
      virtual void loadFile(const std::string &fn, int syntaxErrorsAllowed = 0) = 0;

      virtual void parseCmdLine(int &argc, char **argv) = 0;

      virtual void setValue (const std::string &nm, const std::string &v) = 0;
      virtual void setBool  (const std::string &nm, bool v) = 0;
      virtual void setInt   (const std::string &nm, int v) = 0;
      virtual void setUInt  (const std::string &nm, unsigned v) = 0;
      virtual void setDouble(const std::string &nm, double v) = 0;
      virtual void setInt64  (const std::string &nm, int64_t v) = 0;
      virtual void setUInt64 (const std::string &nm, uint64_t v) = 0;

      virtual std::string getValue (const std::string &nm, const std::string &def) const = 0;
      virtual bool        getBool  (const std::string &nm, bool def) const = 0;
      virtual int         getInt   (const std::string &nm, int def) const = 0;
      virtual unsigned    getUInt  (const std::string &nm, unsigned def) const = 0;
      virtual double      getDouble(const std::string &nm, double def) const = 0;
      virtual int64_t     getInt64  (const std::string &nm, int64_t def) const = 0;
      virtual uint64_t    getUInt64 (const std::string &nm, uint64_t def) const = 0;

      virtual std::string getValue (const std::string &nm) const = 0;
      virtual bool        getBool  (const std::string &nm) const = 0;
      virtual int         getInt   (const std::string &nm) const = 0;
      virtual unsigned    getUInt  (const std::string &nm) const = 0;
      virtual double      getDouble(const std::string &nm) const = 0;
      virtual int64_t     getInt64  (const std::string &nm) const = 0;
      virtual uint64_t    getUInt64 (const std::string &nm) const = 0;

      static std::string getHostName();

      /**
       * Returns the full path of the binary (the .exe file)
       */
      static std::string getBinaryPath();

  protected:
    ConfigOptions();
    virtual ~ConfigOptions();
  };

}

#endif // ! FANCYBASE_CONFIG_H
