/*
 * fancydatachannel.h
 *
 *  Created on: 08.05.2009
 *      Author: wglas
 */

#ifndef FANCYDATACHANNEL_H_
#define FANCYDATACHANNEL_H_

#include <fancydatastream.h>
#include <fancychannel.h>

namespace Fancy {

class FANCYBASE_API DataChannel: public virtual Fancy::DataStream {

    Channel::ptr_t channel;

protected:
    DataChannel();

public:
    typedef ObjPtr<DataChannel> ptr_t;

    static ptr_t newInstance();
    static ptr_t newInstance(const Channel::ptr_t &_channel);

    Channel::ptr_t getChannel() const;
    void setChannel(const Channel::ptr_t &_channel);

    // implemented methods of Fancy::DataStream
    bool    readBoolean();
    int8_t  readByte();
    int16_t readShort();
    int32_t readInt();
    int64_t readLong();
    wchar_t readChar();
    float   readFloat();
    double  readDouble();
    std::string readUTF();
    std::string readUTFLong();

    void writeBoolean(bool b);
    void writeByte   (int8_t i);
    void writeShort  (int16_t i);
    void writeInt    (int32_t i);
    void writeLong   (int64_t i);
    void writeChar   (wchar_t c);
    void writeFloat  (float f);
    void writeDouble (double d);
    void writeUTF    (const char *s, int l=-1);
    void writeUTFLong(const char *s, int l=-1);

    void flush();
    void close();
    void shutdown(ShutdownOption o);
    void putAuthenticationInfo(Fancy::ObjPtr<Fancy::Map> &metadata) const;

protected:
    virtual ~DataChannel();
};

}

#endif /* FANCYDATACHANNEL_H_ */
