/*
 * fancylist.h
 *
 *  Created on: 13.05.2009
 *      Author: wglas
 */

#ifndef FANCYLIST_H_
#define FANCYLIST_H_

#include <fancycollection.h>
#include <fancyarray.h>
#include <fancyserializable.h>

namespace Fancy {

class Iterator;

/*!
 * An interface for an untyped list of objects.
 * This is pretty much like \c java.util.List and is used in favor
 * to a template in order to avoid code-bloat and to allow a generic
 * deserializer to be implemented.
 */
class FANCYBASE_API List: public virtual Collection {
public:
    typedef ObjPtr<List> ptr_t;

    List() {}

    virtual size_t getSize() const = 0;

    virtual ObjPtr<Object> get(size_t i) const = 0;

    virtual void set(size_t i, const ObjPtr<Object> &o) = 0;

    virtual ObjPtr<Iterator> getBeginIterator();
    virtual ObjPtr<Iterator> getEndIterator();

    virtual bool isGrowable() const;

    // optional operations.
    virtual void add(const ObjPtr<Object> &o);
    virtual ObjPtr<Object> remove(size_t i);
    virtual ObjPtr<Object> pop();
    virtual void clear();

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    static ObjPtr<List> newInstance();
    static ObjPtr<List> newInstance(size_t reserve);
    static ObjPtr<List> newInstance(getSerialID_s_t getSerialID_s, getSerialVersionUID_s_t getSerialVersionUID_s);
    static ObjPtr<List> newInstance(getSerialID_s_t getSerialID_s, getSerialVersionUID_s_t getSerialVersionUID_s, size_t reserve);

    template <class T> static ObjPtr<List> newTypedInstance() {
        return List::newInstance(T::getSerialID_s,T::getSerialVersionUID_s);
    }

    template <class T> static ObjPtr<List> newTypedInstance(size_t reserve) {
        return List::newInstance(T::getSerialID_s,T::getSerialVersionUID_s,reserve);
    }

protected:
    virtual ~List();
};

typedef ObjPtr<List> (*newArrayObjectListInstance_t)(size_t);

/*!
 * This class adapts the fixed-size ArrayObject template to the untyped List interface,
 * which is mainly needed in order to implement a generic deserializer.
 */
template <class T> class ObjectArray : public ArrayObject<ObjPtr<T> >, public List {

public:
    typedef ObjPtr<ObjectArray<T> > ptr_t;
    typedef ArrayObject<ObjPtr<T> > super_t;

    static ptr_t newInstance(size_t sz)
    { return new ObjectArray(sz); }

    static ObjPtr<List> newListInstance(size_t sz)
    { return new ObjectArray(sz); }

    const char *getComponentSerialID()     const { return T::getSerialID_s(); }
    int64_t getComponentSerialVersionUID() const { return T::getSerialVersionUID_s();}

    size_t getSize() const { return super_t::getSize(); }

    ObjPtr<Object> get(size_t i) const { return ObjPtr<Object>(*super_t::elem_p(i)); }

    void set(size_t i, const ObjPtr<Object> &o) { *super_t::elem_p(i) = o.check_cast<T>(); }

    bool lessThan(const Object &o) const {
        return List::lessThan(o);
    }

    bool equals(const Object &o) const {
        return List::equals(o);
    }

protected:
    ObjectArray(size_t sz) :
        ArrayObject<ObjPtr<T> >(sz) {}
   ~ObjectArray() {}
};

}

#endif /* FANCYLIST_H_ */
