/*
 * fancynumbers.h
 *
 *  Created on: 17.05.2009
 *      Author: wglas
 */

#ifndef FANCYNUMBERS_H_
#define FANCYNUMBERS_H_

#include <fancyserializable.h>

namespace Fancy {

class FANCYBASE_API Number: public Serializable {
public:
    typedef ObjPtr<Number> ptr_t;

    Number() {}

    virtual int8_t  byteValue()   const = 0;
    virtual int16_t shortValue()  const = 0;
    virtual int32_t intValue()    const = 0;
    virtual int64_t longValue()   const = 0;
    virtual float   floatValue()  const = 0;
    virtual double  doubleValue() const = 0;

    // these functions are here in order to fulfill the requirements of
    // ArrayObjectList<Number> upon serialization and deserialization.
    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();

protected:
    virtual ~Number();
};

class FANCYBASE_API Byte: public Number {

    int8_t v;

public:
    typedef ObjPtr<Byte> ptr_t;

    static ObjPtr<Byte>   newInstance(const std::string &s);
    static ObjPtr<Byte>   newInstance(int8_t _v);
    static ObjPtr<Serializable> newSerializableInstance();

    static int8_t toByte(const Object::ptr_t &obj);

    int8_t  byteValue()   const;
    int16_t shortValue()  const;
    int32_t intValue()    const;
    int64_t longValue()   const;
    float   floatValue()  const;
    double  doubleValue() const;

    std::string toString() const;

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    // implementation of Fancy::Serializable
    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;
    void writeFields(const ObjPtr<Serializer> &serializer) const;
    ObjPtr<Serializable> readFields(const ObjPtr<Serializer> &serializer, int64_t serialVersionUID);

protected:
    Byte(int8_t _v) :v(_v) {}
    ~Byte();
};

class FANCYBASE_API Short: public Number {

    int16_t v;

public:
    typedef ObjPtr<Short> ptr_t;

    static ObjPtr<Short>   newInstance(const std::string &s);
    static ObjPtr<Short>   newInstance(int16_t _v);
    static ObjPtr<Serializable> newSerializableInstance();

    static int16_t toShort(const Object::ptr_t &obj);

    int8_t  byteValue()   const;
    int16_t shortValue()  const;
    int32_t intValue()    const;
    int64_t longValue()   const;
    float   floatValue()  const;
    double  doubleValue() const;

    std::string toString() const;

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    // implementation of Fancy::Serializable
    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;
    void writeFields(const ObjPtr<Serializer> &serializer) const;
    ObjPtr<Serializable> readFields(const ObjPtr<Serializer> &serializer, int64_t serialVersionUID);

protected:
    Short(int16_t _v) :v(_v) {}
    ~Short();
};

class FANCYBASE_API Integer: public Number {

    int32_t v;

public:
    typedef ObjPtr<Integer> ptr_t;

    static ObjPtr<Integer> newInstance(const std::string &s);
    static ObjPtr<Integer> newInstance(int32_t _v);
    static ObjPtr<Serializable> newSerializableInstance();

    static int32_t toInteger(const Object::ptr_t &obj);

    int8_t  byteValue()   const;
    int16_t shortValue()  const;
    int32_t intValue()    const;
    int64_t longValue()   const;
    float   floatValue()  const;
    double  doubleValue() const;

    std::string toString() const;

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    // implementation of Fancy::Serializable
    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;
    void writeFields(const ObjPtr<Serializer> &serializer) const;
    ObjPtr<Serializable> readFields(const ObjPtr<Serializer> &serializer, int64_t serialVersionUID);

protected:
    Integer(int32_t _v) :v(_v) {}
    ~Integer();
};

class FANCYBASE_API Long: public Number {

    int64_t v;

public:
    typedef ObjPtr<Long> ptr_t;

    static ObjPtr<Long>   newInstance(const std::string &s);
    static ObjPtr<Long>   newInstance(int64_t _v);
    static ObjPtr<Serializable> newSerializableInstance();

    static int64_t toLong(const Object::ptr_t &obj);

    int8_t  byteValue()   const;
    int16_t shortValue()  const;
    int32_t intValue()    const;
    int64_t longValue()   const;
    float   floatValue()  const;
    double  doubleValue() const;

    std::string toString() const;

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    // implementation of Fancy::Serializable
    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;
    void writeFields(const ObjPtr<Serializer> &serializer) const;
    ObjPtr<Serializable> readFields(const ObjPtr<Serializer> &serializer, int64_t serialVersionUID);

protected:
    Long(int64_t _v) :v(_v) {}
    ~Long();
};

class FANCYBASE_API Float: public Number {

    float v;

public:
    typedef ObjPtr<Float> ptr_t;

    static ObjPtr<Float>   newInstance(const std::string &s);
    static ObjPtr<Float>   newInstance(float _v);
    static ObjPtr<Serializable> newSerializableInstance();

    static float toFloat(const Object::ptr_t &obj);

    int8_t  byteValue()   const;
    int16_t shortValue()  const;
    int32_t intValue()    const;
    int64_t longValue()   const;
    float   floatValue()  const;
    double  doubleValue() const;

    std::string toString() const;

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    // implementation of Fancy::Serializable
    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;
    void writeFields(const ObjPtr<Serializer> &serializer) const;
    ObjPtr<Serializable> readFields(const ObjPtr<Serializer> &serializer, int64_t serialVersionUID);

protected:
    Float(float _v) :v(_v) {}
    ~Float();
};

class FANCYBASE_API Double: public Number {

    double v;

public:
    typedef ObjPtr<Double> ptr_t;

    static ObjPtr<Double>   newInstance(const std::string &s);
    static ObjPtr<Double>   newInstance(double _v);
    static ObjPtr<Serializable> newSerializableInstance();

    static double toDouble(const Object::ptr_t &obj);

    int8_t  byteValue()   const;
    int16_t shortValue()  const;
    int32_t intValue()    const;
    int64_t longValue()   const;
    float   floatValue()  const;
    double  doubleValue() const;

    std::string toString() const;

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    // implementation of Fancy::Serializable
    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;
    void writeFields(const ObjPtr<Serializer> &serializer) const;
    ObjPtr<Serializable> readFields(const ObjPtr<Serializer> &serializer, int64_t serialVersionUID);

protected:
    Double(double _v) :v(_v) {}
    ~Double();
};

class FANCYBASE_API Boolean: public Serializable {

    bool v;

public:
    typedef ObjPtr<Boolean> ptr_t;

    static ObjPtr<Boolean> newInstance(const char *s);
    static ObjPtr<Boolean> newInstance(const std::string &s);
    static ObjPtr<Boolean> newInstance(bool _v);
    static ObjPtr<Serializable>  newSerializableInstance();

    static bool toBoolean(const Object::ptr_t &obj);

    bool booleanValue()   const;

    std::string toString() const;

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    // implementation of Fancy::Serializable
    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;
    void writeFields(const ObjPtr<Serializer> &serializer) const;
    ObjPtr<Serializable> readFields(const ObjPtr<Serializer> &serializer, int64_t serialVersionUID);

protected:
    Boolean(bool _v) :v(_v) {}
    ~Boolean();
};

class FANCYBASE_API Character: public Serializable {

    wchar_t v;

public:
    typedef ObjPtr<Character> ptr_t;

    static ObjPtr<Character> newInstance(const std::string &s);
    static ObjPtr<Character> newInstance(wchar_t _v);
    static ObjPtr<Serializable>  newSerializableInstance();

    static wchar_t toCharacter(const Object::ptr_t &obj);

    wchar_t charValue() const;

    std::string toString() const;

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    // implementation of Fancy::Serializable
    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;
    void writeFields(const ObjPtr<Serializer> &serializer) const;
    ObjPtr<Serializable> readFields(const ObjPtr<Serializer> &serializer, int64_t serialVersionUID);

protected:
    Character(wchar_t _v) :v(_v) {}
    ~Character();
};
}

#endif /* FANCYNUMBERS_H_ */
