/*
 * fancysessioncontextfactory.h
 *
 *  Created on: 09.12.2009
 *      Author: wglas
 */

#ifndef FANCYSESSIONCONTEXTFACTORY_H_
#define FANCYSESSIONCONTEXTFACTORY_H_

#include <fancyobject.h>

namespace Fancy {

/**
 * A session context factory is used to instantiate a thread-bound context from
 * a possibly pooled resource like database connections or remoting clients.
 */
class FANCYBASE_API SessionContextFactory: public virtual Fancy::Object {
public:

    typedef ObjPtr<SessionContextFactory> ptr_t;

    /**
     * @return The key which is used to bind a session context to the current thread
     *         using @c Fancy::Thread::setSpecific() inside @c newSessionContext().
     */
    virtual Fancy::Object::ptr_t getSpecificKey() const = 0;

    /**
     * @return A new session context. The context is usually bound to the current thread
     *         using the key returned by @c getSpecificKey(). Pooled resources are pulled from
     *         the appropriate pool.
     *         If the returned object is destroyed, the session context is closed,
     *         the local context is unbound from the current thread and pooled
     *         resources are returned to the client pool.
     */
    virtual Fancy::Object::ptr_t newSessionContext() = 0;

protected:
    SessionContextFactory();
    virtual ~SessionContextFactory();
};

}

#endif /* FANCYSESSIONCONTEXTFACTORY_H_ */
