/*
 * fancythreadpool.h
 *
 *  Created on: 27.07.2009
 *      Author: wglas
 */

#ifndef FANCYTHREADPOOL_H_
#define FANCYTHREADPOOL_H_

#include <fancyobject.h>

namespace Fancy {

class Runnable;

class FANCYBASE_API ThreadPool: public virtual Fancy::Object {
protected:
    ThreadPool();
    virtual ~ThreadPool();
public:
    typedef ObjPtr<ThreadPool> ptr_t;

    /**
     * Return a new default implementation, which holds a minimal number of threads and
     * has an upper limit of threads. Idle threads are suspended after a given timeout.
     *
     * @param minThreads The minimal number of worker threads.
     * @param maxThreads The maximal number of worker threads.
     * @param maxWait The maximal number of waiting runnables, if @c maxThreads workers are busy.
     * @param timeout The timeout in milliseconds after which a thread is suspended.
     * @return An implementation of @c TheadPool.
     */
    static ptr_t newInstance(unsigned minThreads, unsigned maxThreads, unsigned maxWait, unsigned timeout);

    /**
     * Start accepting new runnables. This might incur starting of worker threads
     * depending on your implementation.
     */
    virtual void start() = 0;

    /**
     * Stop accepting new runnables and wait for all other runnables to finish.
     * All currently running threads will be joined after this operation.
     *
     * If not called explicitly, the functionality of this function is called from
     * within the destructor. Long-running destructor code is tedious to debug, so
     * calling this function explicitly is strongly encouraged.
     */
    virtual void stop() = 0;

    /**
     * Schedule the given runnable for execution.
     */
    virtual void execute(const ObjPtr<Runnable> &runnable) = 0;
};

}

#endif /* FANCYTHREADPOOL_H_ */
